; Boards/AT91/DefinesH.s
; Defines various constants and macros for the AT91 board
;
; 24/Dec/1999 ned

; KEEP CONSISTENT WITH BOARDDEFINES.h


; Number of handlers for each exception
UndefinedExcptHandlers	EQU	2
SWIExcptHandlers	EQU	2
PAbortExcptHandlers	EQU	2
DAbortExcptHandlers	EQU	2
IRQExcptHandlers	EQU	2
FIQExcptHandlers	EQU	2

; Constants - Exception related
ResetV		EQU	0
UndefV		EQU	1
SWIV		EQU	2
PAbortV		EQU	3
DAbortV		EQU	4
IRQV		EQU	6
FIQV		EQU	7

; Sizes of the processor mode stacks
IRQ32StackSize		EQU	4096
FIQ32StackSize		EQU	1024
Abort32StackSize	EQU	1024
Undef32StackSize	EQU	1024
SVC32StackSize		EQU	8192
BOARD_TOTALSTKSSIZE	EQU	IRQ32StackSize + FIQ32StackSize + Abort32StackSize + Undef32StackSize + SVC32StackSize

; Location of exception handlers
BOARD_EXCEPTHLOCATION		EQU	0x0
; Location of HAL r/w data
BOARD_HALZPLOCATION		EQU	0xc0
; Location of UART ctrl blks
BOARD_HALZP_UARTSPACE		EQU	0x800				; 2k
; Location of L1 MMU page tables (NOT USED)
; BOARD_ARMMMU_L1PAGETABLES	EQU	0x4000				; 16k (16k of L1 page tables)
	[ :DEF: STANDALONE
; Where we put the processor stacks (NOT IN ZERO PAGE)
BOARD_HALSTKSLOCATION		EQU	0x0207c000			; End of DRAM (16k)
; Total size of zero page
BOARD_HALZP_SIZEOF		EQU	0x1000				; 4k
	|
; Where we put the processor stacks
BOARD_HALSTKSLOCATION		EQU	0x8000				; 32k (16k)
; Total size of zero page
BOARD_HALZP_SIZEOF		EQU	0x10000				; 64k
	]

; Locations of UART zero page locations
ANGELUARTID	EQU	0
HALZP_ANGELUARTSPACE		EQU	(ANGELUARTID * HALUARTBLK_SIZEOF)
DEMONUARTID	EQU	1
HALZP_DEMONUARTSPACE		EQU	(DEMONUARTID * HALUARTBLK_SIZEOF)
AT91AUARTID	EQU	2
AT91AUARTADDR	EQU	0xFFFD0000
HALZP_AT91AUARTSPACE		EQU	(AT91AUARTID * HALUARTBLK_SIZEOF)
AT91BUARTID	EQU	3
AT91BUARTADDR	EQU	0xFFFCC000
HALZP_AT91BUARTSPACE		EQU	(AT91BUARTID * HALUARTBLK_SIZEOF)

; debug macro to set leds
	MACRO
	BOARD_SETLEDS	$reg
	LDR	R1,=0xffff0000
	MOV	R0,#7
	STR	R0,[R1,#0x16]		; enable outputs
	MOV	R0,#$reg
	STR	R0,[R1,#0x30]		; set on
	EOR	R0,R0,#7
	STR	R0,[R1,#0x34]		; set off
	MEND

; This macro is called to remap ROM to its proper place and offset the PC if necessary
; All regs are free
	MACRO
	BOARD_MOVEROM
	; Straight from ARM's own example code
	ADR	R10,EBIinitTable
	MOVS	R0,PC,LSR #20
	MOVEQ	R10,R10,LSL #12
	MOVEQ	R10,R10,LSR #12
	LDR	R12,=BoardMoveRomRemappedAddr
	LDMIA	R10!,{R0-R9,R11}
	STMIA	R11!,{R0-R9}
	MOV	PC,R12
EBIinitTable
; 32,768 MHz master clock assumed
            DCD         0x01002535  ; 0x01000000, 16MB, 2 hold, 16 bits, 6 WS
            DCD         0x02002121  ; 0x02000000, 16MB, 0 hold, 16 bits, 1 WS
            DCD         0x20000000  ; unused
            DCD         0x30000000  ; unused
            DCD         0x40000000  ; unused
            DCD         0x50000000  ; unused
            DCD         0x60000000  ; unused
            DCD         0x70000000  ; unused
            DCD         0x00000001  ; REMAP command
            DCD         0x00000006  ; 7 memory regions, standard read
            DCD         0xffe00000  ; EBI Base address

BoardMoveRomRemappedAddr
	; Jumps here, now at 0x01010000+
	; Copy vectors down
	MOV	R0,#0
	ADR	R1,VectorsStart
	ADR	R2,VectorsEnd
1
	LDR	R3,[R1],#4
	STR	R3,[R0],#4
	CMP	R1,R2
	BLO	%b01

	BOARD_SETLEDS 5

	MEND

; This macro is called to initialise DRAM for use (enable power to it, whatever)
	MACRO
	BOARD_INITIALISEDRAM

	MEND

; This macro is called to return the location of the zero page memory
; NOTE: This gets called in the general ISR so make it quick!
; It returns a pointer to 64k of zero page memory in R0
	MACRO
	BOARD_RETURNZPLOCATION $reg
	[ :DEF: STANDALONE
	; Place it at 0x0, where the onchip SRAM lives
	MOV	$reg,#0
	|
	; Place it 64k before memory end
	LDR	$reg,=(0x02080000 - 0x10000)		; 64k before mem end
	]
	MEND

; This macro is called to set up the memory system of the board
; This may entail: (i) determining memory configuration (ii) building page tables appropriately
; HOWEVER: if no MMU is present, or not desired, then the processor caches may not be enabled
	MACRO
	BOARD_SETUPMEMORYSYSTEM
	BOARD_SETLEDS 4

; --- Determine memory configuration
	; This board has a fixed memory config (AT91, 512Kb)

; --- Build page tables
	;BOARD_RETURNZPLOCATION R9
	;ADD	R0,R9,#BOARD_ARMMMU_L1PAGETABLES
	;BL	ARMMMU_BeginL1PageTableBuild

	[ 1=0
	; Map 0x00000000 to 0x00200000 to same (ROM)
	MOV	R1,#0				; start Mb no
	MOV	R2,#0				; start desired virtual addr
	MOV	R3,#0x00200000			; end desired virtual addr
	MOV	R4,#0xa				; domain, access permissions, caching & buffering of section
	BL	ARMMMU_MakeL1PageTableEntries
	
	; Map 0x00200000 to 0x08000000 to Fault
	MOV	R3,#0x08000000			; end desired virtual addr
	MOV	R4,#0				; domain of section
	BL	ARMMMU_MakeL1PageTableEntries
	
	; Map 0x08000000 to 0x08200000 to 0xc0000000 (DRAM)
	MOV	R2,#0x08000000			; start desired virtual addr
	MOV	R3,#0x08200000			; end desired virtual addr
	MOV	R4,#0				; domain of section
	BL	ARMMMU_MakeL1PageTableEntries

	; Map 0x08200000 to 0x00000000 to Fault
	MOV	R3,#0x00000000			; end desired virtual addr
	MOV	R4,#0				; domain of section
	BL	ARMMMU_MakeL1PageTableEntries

	; Set the MMU to use these tables
	MOV	R2,R0
	LDR	R0,=ARMMMU_SETTTB :OR: ARMMMU_SETMMU :OR: ARMMMU_SETWB :OR: ARMMMU_SETROM
	LDR	R1,=ARMMMU_SETTTB :OR: ARMMMU_SETMMU :OR: ARMMMU_SETWB :OR: ARMMMU_SETROM
	BL	ARMMMU_Set
	]

; --- Enable caches
	LDR	R0,=ARMCACHE_SETUCACHE
	LDR	R1,=ARMCACHE_SETUCACHE
	BL	ARMCache_Set

; --- Set location and extent of main memory when MMU is enabled
; On AT91 base of memory is 0x02000000 and it extends 512Kb less 64k or 16k depending on debugger presence
	MOV	R0,#0x02000000
	STR	R0,[R9,#HALZP_DRAMBASE]
	MOV	R0,#0x02080000
	[ :DEF: STANDALONE
	SUB	R0,R0,#0x4000			; 16k (for the stacks as zero page is in SRAM)
	|
	SUB	R0,R0,#0x10000			; 64k (entire HAL zero page)
	]
	STR	R0,[R9,#HALZP_DRAMTOP]

	MEND

; This macro is called to disable an unrecognised interrupt
; You have R0-R5 to play with
	MACRO
	BOARD_DISABLEUNKNOWNINTERRUPT

	MEND

; This macro is called to do any board-specific initialisations
; It is called just prior to calling NedHAL_Entry
; Keep within APCS regulations
	MACRO
	BOARD_INITIALISE
	BOARD_SETLEDS 6

	IMPORT	AT91Timer_Initialise
	BL	AT91Timer_Initialise

	IMPORT	AT91UARTDriver_Initialise
	IMPORT	AT91UARTDriver_Finalise
	IMPORT	AT91UARTDriver_WriteN
	IMPORT	AT91UARTDriver_ReadN

	B	AT91DbgIO_After
AT91DbgIO_SetBlk
	DCD	1						; already initialised
	DCD	AT91UARTDriver_Initialise
	DCD	AT91UARTDriver_Finalise
	DCD	AT91UARTDriver_WriteN
	DCD	AT91UARTDriver_ReadN
AT91DbgIO_After
	; Initialise the UART driver to use port B
	ADD	R4,R9,#BOARD_HALZP_UARTSPACE
	ADD	R4,R4,#HALZP_AT91BUARTSPACE
	MOV	R0,R4
	LDR	R1,=AT91BUARTADDR
	MOV	R2,#AT91BUARTID
	BL	AT91UARTDriver_Initialise
	
	; Point DbgIO at the AT91 UART driver
	ADR	R0,AT91DbgIO_SetBlk
	MOV	R1,R4
	BL	DbgIO_Set

	; Set spurious interrupt vector
	B	AT91SpurInt_Skip
AT91SpurInt_Entry
	SUB	R14,R14,#4
	STMFD	R13!,{R14}
	LDR	R14,=0xfffff000
	STR	R0,[R14,#0x130]					; Ack interrupt
	LDMFD	R13!,{PC}^
AT91SpurInt_Skip
	LDR	R0,=0x78787878
	LDR	R1,=0xfffff000
	STR	R0,[R1,#0x134]					; spurious interrupt reg

	BOARD_SETLEDS 2

	MEND

; This macro is called to do any board-specific shutdowns
; It is called when NedHAL_Entry returns
; Keep within APCS regulations
	MACRO
	BOARD_FINALISE
	IMPORT	AT91Timer_Finalise
	BL	AT91Timer_Finalise

	MEND


	END
