; Core/ARM/3/ARMCore.s
; ARMCore module for v3 ARM architectures
;
; 29/Sep/1999 ned

	AREA	ARMCore, CODE, READONLY

	; Include ARMCore constants
	INCLUDE ArmCoreH.s

	; Include zero page map
	INCLUDE Core/ARM/CommonH.s

	; Exports from this module
	EXPORT	ARMCore_Initialise
	EXPORT	ARMCore_Finalise
	EXPORT	ARMCore_Set
	EXPORT	ARMCore_Enable32
	EXPORT	ARMCore_Enable26

ARMCore_Initialise
	BOARD_RETURNZPLOCATION R3
	MRC	p15, 0, R0, c0, c0, 0
	STR	R0, [R3, #HALZP_ARMCORE_CPUTYPE]
	AND	R0,R0,#0x00ff0000
	CMP	R0,#0x00				; = v3 of ARM
	CMPNE	R0,#0x44				; reported by ARMulator
	BNE	ARMCore_Initialise_NotSuitable

	; Set control reg to reasonable value
	LDR	R0,=0x187d
	STR	R0, [R3, #HALZP_ARMCORE_CTRLREG]
	MOV	R0,#0
	MOV	PC, R14
ARMCore_Initialise_NotSuitable
	ADR	R0,ARMCore_Initialise_NotSuitableErrBlk
	MOV	PC,R14
ARMCore_Initialise_NotSuitableErrBlk
	DCB	"ARMCore",0
	DCD	0
	DCD	ARMCore_Initialise_NotSuitableErrBlkMsg
ARMCore_Initialise_NotSuitableErrBlkMsg
	DCB	"Not compatible with local architecture"

ARMCore_Finalise
	MOV	PC,R14

ARMCore_Set
	STMFD	R13!,{R4,R9}
	; R0=bits to set
	; R1=bits to clr
	BOARD_RETURNZPLOCATION R9
	; Ensure interrupts off
	MRS	R4,CPSR
	ORR	R3,R4,#IBit
	MSR	CPSR_all,R3

	LDR	R2,[R9,#HALZP_ARMCORE_CTRLREG]
	BIC	R2,R2,R1
	ORR	R2,R2,R0
	STR	R2,[R9,#HALZP_ARMCORE_CTRLREG]
	MCR	p15, 0, R2, c1, c0, 0

	MSR	CPSR_all,R4
	MOV	R0,#0
	LDMFD	R13!,{R4,R9}
	MOV	PC,R14

ARMCore_Enable32
	MOV	R0,#ARMCORE_SETP32 :OR: ARMCORE_SETD32
	MOV	R1,#0
	B	ARMCore_Set

ARMCore_Enable26
	MOV	R0,#0
	MOV	R1,#ARMCORE_SETP32 :OR: ARMCORE_SETD32
	B	ARMCore_Set

	END
