/* Core/Common.h
Defines common things across all modules

01/Oct/1999 ned
*/

#ifndef _NedHAL_Core_Common_h_
#define _NedHAL_Core_Common_h_

#ifdef __cplusplus
extern "C" {
#endif

/* Always include board defines */
#include "BoardDefines.h"

/* KEEP CONSISTENT WITH CommonH.s */

#define s8 signed char
#define u8 unsigned char
#define s16 signed short int
#define u16 unsigned short int
#define s32 signed long int
#define u32 unsigned long int
#define s64 signed long long int
#define u64 unsigned long long int

/* SWI's available to us */
#define OS_IntOn	0x13
#define OS_IntOff	0x14
#define OS_EnterOS	0x16

/* Keep consistent with CommonH.s */
typedef struct HALError_t
{
	union
	{
		u32 id[1];
		char name[8];
	} module;
	u32 code;
	char *message;
} HALError;

/* Keep consistent with CommonH.s */
typedef struct HALUARTBlk_t
{
	void *UARTAddr;
	u32 id;
	u32 baud;
	u8 bits;
	u8 parity;
	u8 stopbits;

	char *txbuff;
	int txbuffsize;
	char *txbuffi, *txbuffo;

	char *rxbuff;
	int rxbuffsize;
	char *rxbuffi, *rxbuffo;
} HALUARTBlk;

/* Keep consistent with CommonH.s */
typedef struct HALZeroPage_t
{
	u32 scratch;
	u32 ARMCore_CPUType;
	u32 ARMCore_CtrlReg;
	u32 ARMMMU_TTBReg;
	u32 DRAMBase;
	u32 DRAMTop;
	HALUARTBlk *DbgIO_UARTBlk;
	struct HandlerInfo_t
	{
		void (*HandlerAddr)(int a);
		int	parameter;
		int flags;
	};
	struct HandlerInfo_t UndefE_Handlers [UndefinedExcptHandlers];
	struct HandlerInfo_t SWIE_Handlers   [SWIExcptHandlers];
	struct HandlerInfo_t PAbortE_Handlers[PAbortExcptHandlers];
	struct HandlerInfo_t DAbortE_Handlers[DAbortExcptHandlers];
	struct HandlerInfo_t IRQE_Handlers   [IRQExcptHandlers];
	struct HandlerInfo_t FIQE_Handlers   [FIQExcptHandlers];

} HALZeroPage;

/* Define code to provide inter-processor synchronisation */

/* Macro to perform test and set on the ARM (used for implementing semaphores) */
#define CPU_LOADANDSET(addr, setval) (__asm { SWP	setval, setval, [addr] }, setval)


#ifdef __cplusplus
}
#endif

#endif
