/*
*********************************************************************************************************
*                                               uC/OS-II
*                                        The Real-Time Kernel
*
*                        (c) Copyright 1992-1998, Jean J. Labrosse, Plantation, FL
*                                          All Rights Reserved
*
*                                          ARM Specific code
*
* File : OS_CPU.H
* By   : N. Douglas
*********************************************************************************************************
*/

#ifdef  OS_CPU_GLOBALS
#define OS_CPU_EXT
#else
#define OS_CPU_EXT  extern
#endif

/*
*********************************************************************************************************
*                                              DATA TYPES
*                                         (Compiler Specific)
*********************************************************************************************************
*/

typedef unsigned char  BOOLEAN;
typedef unsigned char  INT8U;                    /* Unsigned  8 bit quantity                           */
typedef signed   char  INT8S;                    /* Signed    8 bit quantity                           */
typedef unsigned short INT16U;                   /* Unsigned 16 bit quantity                           */
typedef signed   short INT16S;                   /* Signed   16 bit quantity                           */
typedef unsigned long  INT32U;                   /* Unsigned 32 bit quantity                           */
typedef signed   long  INT32S;                   /* Signed   32 bit quantity                           */
typedef float          FP32;                     /* Single precision floating point                    */
typedef double         FP64;                     /* Double precision floating point                    */

typedef unsigned int   OS_STK;                   /* Each stack entry is 32-bit wide                    */

#define BYTE           INT8S                     /* Define data types for backward compatibility ...   */
#define UBYTE          INT8U                     /* ... to uC/OS V1.xx.  Not actually needed for ...   */
#define WORD           INT16S                    /* ... uC/OS-II.                                      */
#define UWORD          INT16U
#define LONG           INT32S
#define ULONG          INT32U

/* 
*********************************************************************************************************
*                                       Generic ARM processor
*
* Method #1:  Disable/Enable interrupts using simple instructions.  After critical section, interrupts
*             will be enabled even if they were disabled before entering the critical section.
*
* Method #2:  Disable/Enable interrupts by preserving the state of interrupts.  In other words, if 
*             interrupts were disabled before entering the critical section, they will be disabled when
*             leaving the critical section.
*********************************************************************************************************
*/
#define  OS_CRITICAL_METHOD    1
#define	OS_IntOn	0x13
#define OS_IntOff	0x14
#define OS_EnterOS	0x16

#if      OS_CRITICAL_METHOD == 1
#define  OS_ENTER_CRITICAL() { \
				void __swi(OS_IntOff) disable(void); \
				disable(); \
			     }
#define  OS_EXIT_CRITICAL()  { \
				void __swi(OS_IntOn) enable(void); \
				enable(); \
			     }
#endif

#if      OS_CRITICAL_METHOD == 2
#define  OS_ENTER_CRITICAL() OSStackAndDisableInts()	/* Defined in os_cpu_a.s */
#define  OS_EXIT_CRITICAL()  OSRestoreStackedIntState()	/* Defined in os_cpu_a.s */
#endif

/*
*********************************************************************************************************
*                                       Generic ARM processor
*********************************************************************************************************
*/

#define  OS_STK_GROWTH       1                       /* Stack grows from HIGH to LOW memory on 80x86  */

#define  OS_TASK_SW()        OSTaskSw()              /* Defined in os_cpu_a.s */

